﻿// Learn more about F# at http://fsharp.net

open System
open System.IO 
open System.Runtime.Serialization
open System.Xml
open System.Xml.Linq 
open System.Xml.Serialization
open System.Collections 
open System.Collections.Generic
open System.Net 
open System.Text.RegularExpressions
open Microsoft.Search.Interop  
open System.Data

open Resoures 

let RegExprAttrHref = "href\s*=\s*\"([^\"]*?)\""
let RegExprAWithHrefContent = "<\s*a\s+(\s\S[^>])*?href=\"[\s\S]*?\"(\s[^>]*)?>[\s\S]*?<\s*/a\s*>"
let RegExprHttp = "http[s]{0,1}://\s*\S*"
let RegExprInBraces = "\((\s*\S*)?\)"
let RegExprAClose = "<\s*/\s*a\s*>"
let RegExprImg = "<\s*img\s+([\s\S]*?)\s+/\s*>"
let RegExprAspExt = ".asp[x]{0,1}"
let HtmlExt = ".htm"
let SubfolderFiles = "Files"
let ExternalImage = "external.gif"
let ImgTagExternal = "<img class=\"externImage\" src=\"./" + SubfolderFiles + "/" + ExternalImage + "\" alt=\"" + ExternalImage + "\" />"

let curFileHtm = "Problem.htm"
let lstHtmFiles: HashSet<string> = new HashSet<string>(["45h6515x.htm"; "6xfx8bsa.htm"; "0hy4d4bd.htm"; "xxtcf143.htm"; "dzx6zwec.htm"])

let GetHtmlBeforeBody(html: string): string =
    //let regExrBody = "<\s*body(\s[^>]*)?>[\s\S]*?<\s*/body\s*>"
    let reBody = new Regex(CommonDefs.RegExprBody, RegexOptions.IgnoreCase)
    let matchBody = reBody.Match(html)
    if (matchBody.Success) then
        let partHtml = html.Substring(0, matchBody.Index)
        //let regExprHeadClose = "<\s*/\s*head\s*>"
        let reHeadClose = new Regex(CommonDefs.RegExprHeadClose, RegexOptions.IgnoreCase)
        let matchHeadClose = reHeadClose.Match(partHtml)
        if (matchHeadClose.Success) then
            partHtml.Substring(0, matchHeadClose.Index) 
                + CommonDefs.StrNewLine + CommonDefs.StyleExternalImage + CommonDefs.StrNewLine 
                + matchHeadClose.Value 
        else
            partHtml
    else 
         raise <| new NullReferenceException("Regex.Match(html): " + CommonDefs.RegExprBody)

let GetHtmlTopicContainer(html: string): string = 
    if (String.IsNullOrEmpty(html)) then
        html
    else
        //let regExrDivs = "<\s*div\s+\w*class=\"(topicContainer|topicEndLine|footer)\w*\"(\s[^>]*)?>[\s\S]*?<\s*/div\s*>"
        let reDivs = new Regex(CommonDefs.RegExprDivClsTopicContainerOrTopicEndLineOrFooter, RegexOptions.IgnoreCase)
        let matchColDivs = reDivs.Matches(html)
        let lstIndexesDivs = [for m in matchColDivs -> m.Groups.Item(0).Index]
        if (lstIndexesDivs.Length >= 2) then
            let mutable txtHtml = html.Substring(lstIndexesDivs.Head, lstIndexesDivs.Tail.Head - lstIndexesDivs.Head)
            //let regExrScriptWithSrc = "<\s*script\s+\w*src=\"[\s\S]*?\"(\s[^>]*)?>[\s\S]*?<\s*/script\s*>"
            //let regExrScript = "<\s*script(\s[^>]*)?>[\s\S]*?<\s*/script\s*>"
            let reScrips = new Regex(CommonDefs.RegExprScript, RegexOptions.IgnoreCase) //RegExprScriptWithSrc
            let matchColScrips = reScrips.Matches(html)
            for m in matchColScrips do txtHtml <- txtHtml + CommonDefs.StrNewLine + m.Groups.Item(0).Value
            txtHtml
        else
            raise <| new ArgumentOutOfRangeException("GetHtmlTopicContainer(): lstIndexesDivs.Length = " + lstIndexesDivs.Length.ToString() + " != 2") 

let CorrectUrl(purl: string, url: string): string =
    // let regExprHttp = "http[s]{0,1}://\s*\S*"
    if (not <| Regex.Match(url, CommonDefs.RegExprHttp).Success) then 
        let puri = new Uri(purl)
        if (url.[0..1] = "//") then  
            puri.Scheme + ":" + url
        else     
            puri.Scheme + "://" + puri.Host + url
    else url

let ApplyReplaceLstExprs(html: string, lstReplacement: (string*string) list): string =
    let mutable retVal = html
    for (rexpr, text) in lstReplacement do
        retVal <- Regex.Replace(retVal, rexpr, text)
    retVal

let GetListLinks(purl: string, htmlLinksPart: string): (string*string) list =
    if (String.IsNullOrEmpty(htmlLinksPart)) then
        []
    else
        // let regExrLinks = "<\s*a(\s[^>]*)?>[\s\S]*?<\s*/a\s*>"
        let reLinks = new Regex(CommonDefs.RegExprAContent, RegexOptions.IgnoreCase)
        let matchColLinks = reLinks.Matches(htmlLinksPart)
        // let regExrHref = "href\s*=\s*\"([^\"]*)\""
        // let regExrTitle = "title\s*=\s*\"([^\"]*)\""
        [for m in matchColLinks do 
            let mh = Regex.Match(m.Groups.Item(0).Value, CommonDefs.RegExprAttrHref)
            let mt = Regex.Match(m.Groups.Item(0).Value, CommonDefs.RegExprAttrTitle)
            if (mh.Success 
                && not(mh.Groups.Item("1").Captures.Item(0).Value.Contains("#")) 
                && not(String.IsNullOrEmpty(mh.Groups.Item("1").Captures.Item(0).Value)) 
                && mt.Success) then 
                    yield (CorrectUrl(purl, mh.Groups.Item("1").Captures.Item(0).Value),
                            ApplyReplaceLstExprs(mt.Groups.Item("1").Captures.Item(0).Value, 
                                CommonDefs.RegExprLstHtmlRepl))] 

let GetHtmlWithLinks(html: string): string = 
    if (String.IsNullOrEmpty(html)) then
        html
    else
        //let regExrDivs = "<\s*div\s+\w*class=\"(toclevel1\s+\w*|communityContentNavigation)\"(\s[^>]*)?>[\s\S]*?<\s*/div\s*>"
        let reDivs = new Regex(CommonDefs.RegExprDivClsToclevel1OrCommunityContentNavigation, RegexOptions.IgnoreCase)
        let matchColDivs = reDivs.Matches(html)
        let lstIndexesDivs = [for m in matchColDivs -> m.Groups.Item(0).Index]
        if (lstIndexesDivs.Length > 1) then
            html.Substring(lstIndexesDivs.Item(0), lstIndexesDivs.Item(lstIndexesDivs.Length - 1) - lstIndexesDivs.Item(0))
        else if (lstIndexesDivs.Length > 0) then
            matchColDivs.Item(0).Value 
        else
            raise <| new ArgumentOutOfRangeException("GetHtmlWithLinks() : lstIndexesDivs.Length = " + lstIndexesDivs.Length.ToString()) 

let GetHtmlWithNavDivCurrentroot(html: string): string = 
    if (String.IsNullOrEmpty(html)) then
        html
    else
        //let regExprDivClsNavDivCurrentroot = "<\s*div\s+\w*class=\"(nav_div_currentroot)\"(\s[^>]*)?>[\s\S]*?<\s*/div\s*>"
        let reDivs = new Regex(CommonDefs.RegExprDivClsNavDivCurrentroot, RegexOptions.IgnoreCase)
        let matchColDivs = reDivs.Matches(html)
        let lstIndexesDivs = [for m in matchColDivs -> m.Groups.Item(0).Index]
        if (lstIndexesDivs.Length > 1) then
            html.Substring(lstIndexesDivs.Item(0), lstIndexesDivs.Item(lstIndexesDivs.Length - 1) - lstIndexesDivs.Item(0))
        else if (lstIndexesDivs.Length > 0) then
            matchColDivs.Item(0).Value  
        else
            String.Empty 
            //raise <| new ArgumentOutOfRangeException("GetHtmlWithNavDivCurrentroot() : lstIndexesDivs.Length = " + lstIndexesDivs.Length.ToString()) 

let GetHtmlAndLinksFromHtml(source: string, html: string): string*((string*string) list)*((string*string) list) = 
    ((html |> GetHtmlBeforeBody) + "<body>" + (html |> GetHtmlTopicContainer) + "</body></html>",
        GetListLinks(source, GetHtmlWithLinks(html)),
        GetListLinks(source, GetHtmlWithNavDivCurrentroot(html)))

let GetFileContents (file: string): string =
    use sr = File.OpenText(file)
    sr.ReadToEnd()

let fullHtml = GetFileContents(curFileHtm)
let mutable html, lstLinks, lstNavParentLinks = GetHtmlAndLinksFromHtml("http://msdn.microsoft.com/ru-ru/library/dd627203", fullHtml)

ignore()
