﻿module Resoures

open System

type public ModuleResoures() =
    [<DefaultValue>] static val mutable private appResoures: System.Resources.ResourceManager
    
    static member public AppResoures = 
        if (ModuleResoures.appResoures = null) then
            ModuleResoures.appResoures <- new System.Resources.ResourceManager("Resources", System.Reflection.Assembly.GetExecutingAssembly())   
        ModuleResoures.appResoures

    static member public GetAppResourceString(resourceString): string =
        let value = ModuleResoures.AppResoures.GetString(resourceString)
        if (System.String.IsNullOrEmpty(value)) then
            raise <| new ArgumentNullException("AppResoures.GetString(" + resourceString + ")")
        else
            value 

type public CommonDefs() =
    static member public UseOfflineVersion = true
    static member public RegExprAContent = "<\s*a(\s[^>]*)?>[\s\S]*?<\s*/a\s*>"
    static member public RegExprAWithHrefContent = "<\s*a\s+(\s\S[^>])*?href=\"[\s\S]*?\"(\s[^>]*)?>[\s\S]*?<\s*/a\s*>"
    static member public RegExprAWithHrefContentFormat = "<\s*a\s+(\s\S[^>])*?href=\"({0})\"(\s[^>]*)?>[\s\S]*?<\s*/a\s*>"
    static member public RegExprBody = "<\s*body(\s[^>]*)?>[\s\S]*?<\s*/body\s*>"
    static member public RegExprDivClsNavDivCurrentroot = "<\s*div\s+\w*class=\"(nav_div_currentroot)\"(\s[^>]*)?>[\s\S]*?<\s*/div\s*>"
//    static member public RegExprDivClsToclevel1OrCommunityContentNavigation = "<\s*div\s+\w*class=\"(toclevel1\s+\w*|communityContentNavigation|footer)\"(\s[^>]*)?>[\s\S]*?<\s*/div\s*>"
//    static member public RegExprDivClsTopicContainerOrTopicEndLine = "<\s*div\s+\w*class=\"(topicContainer|topicEndLine|footer)\w*\"(\s[^>]*)?>[\s\S]*?<\s*/div\s*>"
    static member public RegExprDivClsToclevel1OrCommunityContentNavigation = "<\s*div\s+\w*class=\"(toclevel1\s+\w*|communityContentNavigation)\"(\s[^>]*)?>[\s\S]*?<\s*/div\s*>"
    static member public RegExprDivClsTopicContainerOrTopicEndLineOrFooter = "<\s*div\s+\w*class=\"(topicContainer|topicEndLine|footer)\w*\"(\s[^>]*)?>[\s\S]*?<\s*/div\s*>"
    static member public RegExprScript = "<\s*script(\s[^>]*)?>[\s\S]*?<\s*/script\s*>"
    //static member public RegExprLink = "<\s*link(\s[^>]*)?>[\s\S]*?<\s*/link\s*>"
    static member public RegExprLink = "<\s*link\s+([\s\S]*?)\s+/\s*>"
    static member public RegExprLinkCanonical = "<\s*link\s+[\s\S]*?rel=\"canonical\"([\s\S]*?)\s+/\s*>"
    static member public RegExprMetaNormalizedUrl = "<\s*meta\s+[\s\S]*?name=\"NormalizedUrl\"([\s\S]*?)\s+/\s*>"
    static member public RegExprAttrHref = "href\s*=\s*\"([^\"]*)\""
    static member public RegExprAttrTitle = "title\s*=\s*\"([^\"]*)\""
    static member public RegExprAttrContent = "content\s*=\s*\"([^\"]*)\""
    static member public RegExprAttrRel = "rel\s*=\s*\"([^\"]*)\""
    static member public RegExprAttrHrefOrSrc = "href\s*=\s*\"([^\"]*)\"|src\s*=\s*\"([^\"]*)\""
    static member public RegExprSrc = "src\s*=\s*\"([^\"]*)\""
    static member public RegExprHttp = "http[s]{0,1}://\s*\S*"
    static member public RegExprAspExt = ".asp[x]{0,1}"
    static member public RegExprLeftPartEq = "\s*\w+\s*="
    static member public RegExprAClose = "<\s*/\s*a\s*>"
    static member public RegExprImg = "<\s*img\s+([\s\S]*?)\s+/\s*>"
    static member public RegExprHeadClose = "<\s*/\s*head\s*>"
    static member public RegExprInBraces = "\((\s*\S*)?\)"
    static member public RegExprQChar = "\?"
    static member public RegExpr2QChar = "\\?"
    static member public RegExprLstHtmlRepl = [("&amp;#39;", "'"); ("&#39;", "'"); ("&lt;", "<"); ("&gt;", ">"); ("&amp;", "&"); ("&quot;", "\"")]
    static member public StrNewLine = Environment.NewLine // "\r\n"
    static member public HtmlExt = ".htm"
    static member public XmlExt = ".xml"
    static member public SubfolderFiles = "Files"
    static member public FileSiteInfo = "siteInfo.xml"
    static member public FileSearch = "search.tmp"
    static member public FileAdditionalLinks = "AdditionalLinks.xml"
    static member public CookieLangName = "CodeSnippetContainerLang"
    static member public StyleExternalImage = "<style type=\"text/css\">.externImage{border-width:0;vertical-align:middle;margin:0}</style>"
    static member public ExternalImage = "external.gif"
    static member public AppImage = "appImage.png"
    static member public InitLogoPicture = "logoPictureBox_Image"
    static member public ImgTagExternal = "<img class=\"externImage\" src=\"./" + CommonDefs.SubfolderFiles + "/" + CommonDefs.ExternalImage + "\" alt=\"" + CommonDefs.ExternalImage + "\" />"
    static member public QuerySelectColumns = "System.ItemName,System.Title"
    static member public BuffSize = 1024 //5120
    static member public MaxFileNameLength = 248 //260
    static member public UseCheckFilesSizeProblem = true
    static member public MaxFilesSizeProblem = 1500
    static member public MaxDepthTocLevel = 1
    static member public HttpSearchPage = "<!DOCTYPE html PUBLIC \"-//W3C//DTD XHTML 1.0 Transitional//EN\" \"http://www.w3.org/TR/xhtml1/DTD/xhtml1-transitional.dtd\">" + Environment.NewLine 
                                        + "<html xmlns=\"http://www.w3.org/1999/xhtml\">"
                                        + "<head><title>" + ModuleResoures.GetAppResourceString("HttpSearchTitle") + "</title><style type=\"text/css\">body{font-family:Arial,Helvetica,sans-serif;font-size:75%;font-style:normal;}table{border-width:0}td{padding-top:1px;padding-bottom:1px}</style>"
                                        + "<meta http-equiv=\"content-type\" content=\"text/html; charset=utf-8\" />"
                                        + "</head>\r\n<body><h2 style=\"color: #3f529c\">" + ModuleResoures.GetAppResourceString("HttpSearchTitle") + "</h2>"
    static member public HttpScrptReplWebtrendsCom = "<script type=\"text/javascript\">" + Environment.NewLine 
                                                    + "//<![CDATA[" + Environment.NewLine 
                                                    + " var wt_nvr_ru = 'WT_NVR_RU';" + Environment.NewLine 
                                                    + " var gFpc = 'WT_FPC';" + Environment.NewLine 
                                                    + " var detectedLocale = 'ru-ru';" + Environment.NewLine 
                                                    + " var gTrackEvents = '0';" + Environment.NewLine 
                                                    + "/*]]>*/" + Environment.NewLine 
                                                    + "</script>"
    static member public HttpJScrptSrcToRemove = ["webtrendsscript.js"; "omni_rsid_msdn.js"]
    static member public HttpSearchPageCountFotmat = "\r\n<div>" + ModuleResoures.GetAppResourceString("HttpSearchPageCount1") + " <strong>{0}</strong> " + ModuleResoures.GetAppResourceString("HttpSearchPageCount2") + "</div>"
    static member public HttpSearchItemsHeader = "\r\n<table>"
    static member public HttpSearchItemFotmat = "<tr><td>{0}</td></tr>"
    static member public HttpSearchItemsFooter = "</table>"
    static member public HttpSearchPageEnd = "</body></http>"
    static member public RootItemRefAdditionally = "#Additionally#"
    static member public NotFileNameParts = ["http://"; ":"; "\?"; "/"]
    static member public HtmlAmp = "&amp;"
