﻿module AboutBox

open System
open System.Drawing 
open System.Windows.Forms
open System.IO
open Resoures

type AboutBox() as this = 
    inherit Form() 

    let mutable components: System.ComponentModel.Container = null
    let mutable tableLayoutPanel: System.Windows.Forms.TableLayoutPanel = null
    let mutable logoPictureBox: System.Windows.Forms.PictureBox = null  
    let mutable labelProductName: System.Windows.Forms.Label = null 
    let mutable labelVersion: System.Windows.Forms.Label = null 
    let mutable labelCopyright: System.Windows.Forms.Label = null
    let mutable labelCompanyName: System.Windows.Forms.Label = null
    let mutable textBoxDescription: System.Windows.Forms.TextBox = null
    let mutable okButton: System.Windows.Forms.Button = null

    do 
        this.InitializeComponent();
        labelProductName.Text <- this.AssemblyProduct;
        labelCopyright.Text <- this.AssemblyCopyright;
        labelCompanyName.Text <- this.AssemblyCompany;
        textBoxDescription.Text <- this.AssemblyDescription;
        let af = ModuleResoures.AppResoures.GetString("AboutFrmt");
        if (af <> null && af.Length > 0) then
            let at: string = this.AssemblyTitle; 
            this.Text <- String.Format(af, at);
        let vf = ModuleResoures.AppResoures.GetString("VersionFrmt");
        if (vf <> null && vf.Length > 0) then
            let av: string = this.AssemblyVersion; 
            labelVersion.Text <- String.Format(vf, av);
(*
        this.Text <- String.Format(ModuleResoures.AppResoures.GetString("AboutFrmt"), this.AssemblyTitle);
        labelVersion.Text <- String.Format(ModuleResoures.AppResoures.GetString("VersionFrmt"), this.AssemblyVersion)
*)

    override this.Dispose(disposing) =
        if (disposing && (match components with null -> false | _ -> true)) then
          components.Dispose()
        base.Dispose(disposing)

    member private this.InitializeComponent() =
        components <- new System.ComponentModel.Container();
        let resources = new System.ComponentModel.ComponentResourceManager(typeof<AboutBox>);
        tableLayoutPanel <- new System.Windows.Forms.TableLayoutPanel();
        logoPictureBox <- new System.Windows.Forms.PictureBox();
        labelProductName <- new System.Windows.Forms.Label();
        labelVersion <- new System.Windows.Forms.Label();
        labelCopyright <- new System.Windows.Forms.Label();
        labelCompanyName <- new System.Windows.Forms.Label();
        textBoxDescription <- new System.Windows.Forms.TextBox();
        okButton <- new System.Windows.Forms.Button();
        tableLayoutPanel.SuspendLayout();
        (logoPictureBox :> System.ComponentModel.ISupportInitialize).BeginInit();
        //((System.ComponentModel.ISupportInitialize)(logoPictureBox)).BeginInit()
        this.SuspendLayout();
        // 
        // tableLayoutPanel
        // 
        resources.ApplyResources(tableLayoutPanel, "tableLayoutPanel");
        tableLayoutPanel.Controls.Add(logoPictureBox, 0, 0);
        tableLayoutPanel.Controls.Add(labelProductName, 1, 0);
        tableLayoutPanel.Controls.Add(labelVersion, 1, 1);
        tableLayoutPanel.Controls.Add(labelCopyright, 1, 2);
        tableLayoutPanel.Controls.Add(labelCompanyName, 1, 3);
        tableLayoutPanel.Controls.Add(textBoxDescription, 1, 4);
        tableLayoutPanel.Controls.Add(okButton, 1, 5);
        tableLayoutPanel.Name <- "tableLayoutPanel";
        // 
        // logoPictureBox
        // 
        resources.ApplyResources(logoPictureBox, "logoPictureBox");
        logoPictureBox.InitialImage <- Resoures.ModuleResoures.AppResoures.GetObject(Path.GetFileNameWithoutExtension(CommonDefs.InitLogoPicture)) :?> System.Drawing.Bitmap;
        logoPictureBox.Image <- Resoures.ModuleResoures.AppResoures.GetObject(Path.GetFileNameWithoutExtension(CommonDefs.AppImage)) :?> System.Drawing.Bitmap;
        logoPictureBox.Name <- "logoPictureBox";
        tableLayoutPanel.SetRowSpan(logoPictureBox, 6);
        logoPictureBox.TabStop <- false;
        // 
        // labelProductName
        // 
        resources.ApplyResources(labelProductName, "labelProductName");
        labelProductName.MaximumSize <- new System.Drawing.Size(0, 17);
        labelProductName.Name <- "labelProductName";
        // 
        // labelVersion
        // 
        resources.ApplyResources(labelVersion, "labelVersion");
        labelVersion.MaximumSize <- new System.Drawing.Size(0, 17);
        labelVersion.Name <- "labelVersion";
        // 
        // labelCopyright
        // 
        resources.ApplyResources(labelCopyright, "labelCopyright");
        labelCopyright.MaximumSize <- new System.Drawing.Size(0, 17);
        labelCopyright.Name <- "labelCopyright";
        // 
        // labelCompanyName
        // 
        resources.ApplyResources(labelCompanyName, "labelCompanyName");
        labelCompanyName.MaximumSize <- new System.Drawing.Size(0, 17);
        labelCompanyName.Name <- "labelCompanyName";
        // 
        // textBoxDescription
        // 
        resources.ApplyResources(textBoxDescription, "textBoxDescription");
        textBoxDescription.Name <- "textBoxDescription";
        textBoxDescription.ReadOnly <- true;
        textBoxDescription.TabStop <- false;
        // 
        // okButton
        // 
        resources.ApplyResources(okButton, "okButton");
        okButton.DialogResult <- System.Windows.Forms.DialogResult.Cancel;
        okButton.Name <- "okButton";
        // 
        // AboutBox
        // 
        this.AcceptButton <- okButton;
        resources.ApplyResources(this, "$this");
        this.AutoScaleMode <- System.Windows.Forms.AutoScaleMode.Font;
        this.Controls.Add(tableLayoutPanel);
        this.FormBorderStyle <- System.Windows.Forms.FormBorderStyle.FixedDialog;
        this.MaximizeBox <- false;
        this.MinimizeBox <- false;
        this.Name <- "AboutBox";
        this.ShowIcon <- false;
        this.ShowInTaskbar <- false;
        tableLayoutPanel.ResumeLayout(false);
        tableLayoutPanel.PerformLayout();
        (logoPictureBox :> System.ComponentModel.ISupportInitialize).EndInit();
        //((System.ComponentModel.ISupportInitialize)(logoPictureBox)).EndInit()
        this.ResumeLayout(false)

    static member private ExecutingAssembly = System.Reflection.Assembly.GetExecutingAssembly()

    member this.AssemblyVersion =
        AboutBox.ExecutingAssembly.GetName().Version.ToString()

    member this.AssemblyTitle =
        let attributes = AboutBox.ExecutingAssembly.GetCustomAttributes(typeof<System.Reflection.AssemblyTitleAttribute>, false);
        if (attributes = null || attributes.Length = 0) then
            System.IO.Path.GetFileNameWithoutExtension(AboutBox.ExecutingAssembly.CodeBase)
        else
            let titleAttribute: System.Reflection.AssemblyTitleAttribute = attributes.[0] :?> System.Reflection.AssemblyTitleAttribute;
            if (titleAttribute.Title.Length > 0) then
                titleAttribute.Title
            else
                ""

    member this.AssemblyDescription = 
        let appDescription = ModuleResoures.AppResoures.GetString("AppDescription");
        if (appDescription <> null && appDescription.Length > 0) then
            appDescription
        else 
            let attributes = AboutBox.ExecutingAssembly.GetCustomAttributes(typeof<System.Reflection.AssemblyDescriptionAttribute>, false);
            if (attributes = null || attributes.Length = 0) then
                ""
            else
                (attributes.[0] :?> System.Reflection.AssemblyDescriptionAttribute).Description
    
    member this.AssemblyProduct = 
        let attributes = AboutBox.ExecutingAssembly.GetCustomAttributes(typeof<System.Reflection.AssemblyProductAttribute>, false);
        if (attributes.Length = 0) then
            ""
        else
            (attributes.[0] :?> System.Reflection.AssemblyProductAttribute).Product

    member this.AssemblyCopyright = 
        let attributes = AboutBox.ExecutingAssembly.GetCustomAttributes(typeof<System.Reflection.AssemblyCopyrightAttribute>, false);
        if (attributes = null || attributes.Length = 0) then
            ""
        else
            (attributes.[0] :?> System.Reflection.AssemblyCopyrightAttribute).Copyright

    member this.AssemblyCompany = 
        let attributes = AboutBox.ExecutingAssembly.GetCustomAttributes(typeof<System.Reflection.AssemblyCompanyAttribute>, false);
        if (attributes = null || attributes.Length = 0) then
            ""
        else
            (attributes.[0] :?> System.Reflection.AssemblyCompanyAttribute).Company
    
    member public this.ResetImage() = 
        use image: System.Drawing.Bitmap = Resoures.ModuleResoures.AppResoures.GetObject(Path.GetFileNameWithoutExtension("app")) :?> System.Drawing.Bitmap
        logoPictureBox.Image <- image;
