﻿module SearchTools

open Resoures
open WebTools
open System.Text.RegularExpressions
open Microsoft.Search.Interop
  
let HighLightQuery(txt: string, userQuery: string): string =
    let mutable retVal = ""
    let reQuery = new Regex(userQuery, RegexOptions.IgnoreCase)
    let matchQuery = reQuery.Matches(txt)
    let txtLen = txt.Length 
    for i = matchQuery.Count - 1 downto 0 do
        let ci = matchQuery.Item(i).Groups.Item(0).Index
        let rp = if (ci + matchQuery.Item(i).Groups.Item(0).Length > txtLen) then txtLen else ci + matchQuery.Item(i).Groups.Item(0).Length
        retVal <- retVal + txt.Substring(0, ci) 
                + "<span style=\"background-color: yellow\">" + matchQuery.Item(i).Groups.Item(0).Value + "</span>"
                + txt.Substring(rp) 
    retVal

let GetQueryContents(html: string, userQuery: string): string =
    let mutable retVal = ""
    let txt = Regex.Replace(Regex.Replace(html, "<[/]{0,1}\s*[\s\S]*?(\s[^>]*)?>", ""), "\s+", " ")
    let reQuery = new Regex(userQuery, RegexOptions.IgnoreCase)
    let matchQuery = reQuery.Matches(txt)
    for item = 0 to matchQuery.Count - 1 do
        // Позиция предшествующего вхождения
        let lp = if (item <> 0) then 
                    matchQuery.Item(item - 1).Groups.Item(0).Index 
                        + matchQuery.Item(item - 1).Groups.Item(0).Value.Length
                 else 0
        // Длина текста предшествующего вхождения
        let cl = matchQuery.Item(item).Groups.Item(0).Index - lp
        // Текст предшествющего вхождения
        let txtLeft = if (cl > 0) then txt.Substring(lp, cl) else txt.Substring(lp)
        // Относительная позиция последнего ограничителя текста предшествющего вхождения
        let lpp = txtLeft.LastIndexOf(".")
        // Абсолютная позиция текста от последнего ограничителя текста в предшествющем вхождении
        let lpc =
            if (lpp <> (-1)) then
                lp + lpp + 1
            else
                lp
        // Абсолютная позиция после текущего вхождения
        let curPos = matchQuery.Item(item).Groups.Item(0).Index 
                    + matchQuery.Item(item).Groups.Item(0).Length
        // Абсолютная позиция текста следующего вхождения
        let rp = if (item <> matchQuery.Count - 1) then 
                    matchQuery.Item(item + 1).Groups.Item(0).Index 
                 else 0
        // Текст до следующего вхождения
        let txtRight = if (rp > 0) then txt.Substring(curPos, rp - curPos) else txt.Substring(curPos)
        // Относительная позиция ограничителя в тексте до следующего вхождении
        let rpp = txtRight.IndexOf(".")
        // Длина текста до ограничителя в тексте до следующего вхождении
        let rpc = 
            if (rpp <> (-1)) then
                curPos + rpp + 1
            else
                curPos   
        // Выбор итогового фрагмента текста
        let txtPart = txt.Substring(lpc, rpc - lpc).Trim()
        if (txtPart.Length > 0) then 
            retVal <- retVal + HighLightQuery(txtPart, userQuery) + "...<br>"
    retVal

let SearchUserQuery(folder: string, fileExt: string, querySelectColumns: string, userQuery: string): (string*string) list list =
    let mutable retVal = []
    let manager = new CSearchManagerClass()
    let catalogManager = manager.GetCatalog("SystemIndex");
    let queryHelper = catalogManager.GetQueryHelper();
    queryHelper.QuerySelectColumns <- querySelectColumns //"System.ItemName,System.Title" //,System.Search.AutoSummary
    //System.Search.AutoSummary,System.Search.ContainerHash,System.Search.Contents,System.Search.EntryID,System.Search.GatherTime,System.Search.HitCount,System.Search.IsClosedDirectory,System.Search.IsFullyContained,System.Search.QueryFocusedSummary,System.Search.QueryFocusedSummaryWithFallback,System.Search.Rank,System.Search.Store,System.Search.UrlToIndex,System.Search.UrlToIndexWithModificationTime" //System.Search.ExtendedProperties
    queryHelper.QueryWhereRestrictions <- " AND scope='file:" + folder + "' AND Contains(System.ItemType, '" + fileExt+  "')"
    let sqlQuery = queryHelper.GenerateSQLFromUserQuery(userQuery + " ")
    use conn = new System.Data.OleDb.OleDbConnection(queryHelper.ConnectionString)
    conn.Open()
    use command = new System.Data.OleDb.OleDbCommand(sqlQuery, conn)
    use wdsResults = command.ExecuteReader()
    if (wdsResults.HasRows) then 
        let fieldCount = wdsResults.FieldCount 
        let mutable rowVal = []
        let mutable strVal = ""
        while (wdsResults.Read()) do
            rowVal <- []
            for nField = 0 to fieldCount - 1 do
                strVal <- null
                try
                    strVal <- wdsResults.GetString(nField)
                with 
                    | _ -> ()
                rowVal <- (wdsResults.GetName(nField).ToString(),
                 (if (strVal = null) then 
                        "" 
                    else
                        strVal.ToString().Trim()))::rowVal
            retVal <- List.rev rowVal::retVal
    List.rev retVal

let GenHtmlForQueryResult(folder: string, userQuery: string, lstSearchResult: (string*string) list list): string =
    let mutable html = CommonDefs.HttpSearchPage + System.String.Format(CommonDefs.HttpSearchPageCountFotmat, lstSearchResult.Length.ToString()) 
    html <- html + CommonDefs.HttpSearchItemsHeader
    for itemResult in lstSearchResult do
        let (ht, hv) = itemResult.Head 
        let (tt, tv) = itemResult.Tail.Head  
        if (not(CommonDefs.FileSearch = hv)) then
            let htmlPage = GetLocalFile(folder, hv, CommonDefs.HtmlExt) |> GetFileContents
            let htmlQueryContents = GetQueryContents(htmlPage, userQuery)
            html <- html + System.String.Format(CommonDefs.HttpSearchItemFotmat, "<a href=\"" + hv + "\">" + tv + "</a>")
            if (not <| System.String.IsNullOrEmpty(htmlQueryContents)) then
                html <- html + System.String.Format(CommonDefs.HttpSearchItemFotmat, htmlQueryContents)
    html <- html + CommonDefs.HttpSearchItemsFooter
    html <- html + CommonDefs.HttpSearchPageEnd
    html

