﻿using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Linq;
using System.Text;
using System.Windows.Forms;

using System.IO;
using System.Xml;
using System.Xml.Serialization;
using System.Runtime.Serialization;

namespace UrlDownLoaderResources
{
    public partial class MainForm : Form
    {
        public MainForm()
        {
            InitializeComponent();
            if (cobLang.SelectedIndex == (-1) && cobLang.Items.Count > 0)
            {
                cobLang.SelectedIndex = 0;
            }
        }

        private void exitToolStripMenuItem_Click(object sender, EventArgs e)
        {
            Close(); 
        }

        private void aboutToolStripMenuItem_Click(object sender, EventArgs e)
        {
            AboutBox aboutBox = new AboutBox();
            aboutBox.ShowDialog(this); 
        }
        
        private void btnSelect_Click(object sender, EventArgs e)
        {
            if (folderBrowserDialog.ShowDialog(this) == System.Windows.Forms.DialogResult.OK)
            {
                txtFolder.Text = folderBrowserDialog.SelectedPath;
                if (Directory.Exists(txtFolder.Text))
                {
                    string fileSiteInfo = null;
                    if (File.Exists(fileSiteInfo = Path.Combine(
                        txtFolder.Text,
                        CommonDefs.FileSiteInfo)))
                    {
                        txtUrl.Text = getUrl(fileSiteInfo);
                    }
                    else if (!String.IsNullOrEmpty(txtUrl.Text))
                    {
                        SaveAsFileSiteInfo(txtUrl.Text, fileSiteInfo);
                    }
                }
            }
        }

        private string getUrl(string fileSiteInfo)
        {
            string retVal = String.Empty;
            try
            {
                if (String.IsNullOrEmpty(fileSiteInfo))
                {
                    throw new ArgumentNullException("fileSiteInfo");
                }
                if (!File.Exists(fileSiteInfo))
                {
                    throw new FileNotFoundException(fileSiteInfo);
                }
                //XmlSerializer ser = new XmlSerializer(typeof(SiteInfo));
                DataContractSerializer ser = new DataContractSerializer(typeof(SiteInfo));
                if (ser == null)
                {
                    throw new NullReferenceException(
                        "ser " + Properties.Resources.PhraseAs + " DataContractSerializer");
                }
                using (Stream reader = new FileStream(fileSiteInfo, FileMode.Open))
                {
                    if (reader == null)
                    {
                        throw new NullReferenceException("FileStream(" + fileSiteInfo + ")");
                    }
                    SiteInfo siteInfo = (SiteInfo)ser.ReadObject(reader);
                    if (siteInfo != null && !String.IsNullOrEmpty(siteInfo.url))
                    {
                        retVal = siteInfo.url;
                    }
                    else
                    {
                        throw new NullReferenceException(
                            "siteInfo  " + Properties.Resources.PhraseAs + " SiteInfo");
                    }
                }
            }
            catch (Exception ex)
            {
                MessageBox.Show(
                    this,
                    ex.ToString(),
                    Properties.Resources.CaptionError,
                    MessageBoxButtons.OK,
                    MessageBoxIcon.Error);
            }
            return retVal;
        }

        private bool SaveAsFileSiteInfo(string url, string fileSiteInfo)
        {
            bool bRetVal = false;
            try
            {
                if (String.IsNullOrEmpty(url))
                {
                    throw new ArgumentNullException("url");
                }
                if (String.IsNullOrEmpty(fileSiteInfo))
                {
                    throw new ArgumentNullException("fileSiteInfo");
                }
                if (File.Exists(fileSiteInfo))
                {
                    File.Delete(fileSiteInfo);
                    /*if (File.Exists(fileSiteInfo))
                    { 
                    }*/
                }
                SiteInfo siteInfo = new SiteInfo(url);
                if (siteInfo == null)
                {
                    throw new NullReferenceException(
                        "siteInfo " + Properties.Resources.PhraseAs + " SiteInfo");
                }
                //XmlSerializer ser = new XmlSerializer(typeof(SiteInfo));
                DataContractSerializer ser = new DataContractSerializer(typeof(SiteInfo));
                if (ser == null)
                {
                    throw new NullReferenceException(
                        "ser " + Properties.Resources.PhraseAs + " DataContractSerializer");
                }
                using (Stream writer = new FileStream(fileSiteInfo, FileMode.Create))
                {
                    if (writer == null)
                    {
                        throw new FileNotFoundException(fileSiteInfo);
                    }
                    ser.WriteObject(writer, siteInfo);
                    bRetVal = true;
                }
            }
            catch (Exception ex)
            {
                MessageBox.Show(
                    this,
                    ex.ToString(),
                    Properties.Resources.CaptionError,
                    MessageBoxButtons.OK,
                    MessageBoxIcon.Error);
            }
            return bRetVal;
        }

        private void tabControl_SelectedIndexChaneged(object sender, EventArgs e)
        {

        }

        private void btnBrowse_Click(object sender, EventArgs e)
        {

        }

        private void testToolStripMenuItem_Click(object sender, EventArgs e)
        {

        }
    }
}
